// logger.js
const fs = require('fs');
const path = require('path');

class Logger {
  constructor(options = {}) {
    this.options = {
      logToConsole: options.logToConsole !== false,
      logToFile: options.logToFile || false,
      logDirectory: options.logDirectory || path.join(__dirname, '..', '..', 'logs'),
      userActivityLogFile: options.userActivityLogFile || 'user-activity.log',
      apiLogFile: options.apiLogFile || 'api.log',
      // Log levels: debug, info, warn, error
      minLevel: options.minLevel || 'info'
    };

    // Create log directory if it doesn't exist and logging to file is enabled
    if (this.options.logToFile) {
      if (!fs.existsSync(this.options.logDirectory)) {
        fs.mkdirSync(this.options.logDirectory, { recursive: true });
      }
    }

    // Log levels and their priorities
    this.levels = {
      debug: 0,
      info: 1,
      warn: 2,
      error: 3
    };
  }

  shouldLog(level) {
    return this.levels[level] >= this.levels[this.options.minLevel];
  }

  formatLogEntry(type, message, data = {}) {
    const timestamp = new Date().toISOString();
    const logObject = {
      timestamp,
      type,
      message
    };

    if (Object.keys(data).length > 0) {
      logObject.data = data;
    }

    return JSON.stringify(logObject);
  }

  writeToFile(content, isUserActivity = false) {
    if (!this.options.logToFile) return;

    const logFile = isUserActivity 
      ? path.join(this.options.logDirectory, this.options.userActivityLogFile) 
      : path.join(this.options.logDirectory, this.options.apiLogFile);

    fs.appendFile(logFile, content + '\n', (err) => {
      if (err) {
        console.error(`Error writing to log file: ${err.message}`);
      }
    });
  }

  debug(message, data = {}) {
    if (!this.shouldLog('debug')) return;
    
    const logEntry = this.formatLogEntry('DEBUG', message, data);
    
    if (this.options.logToConsole) {
      console.debug(logEntry);
    }
    
    this.writeToFile(logEntry);
  }

  info(message, data = {}) {
    if (!this.shouldLog('info')) return;
    
    const logEntry = this.formatLogEntry('INFO', message, data);
    
    if (this.options.logToConsole) {
      console.log(logEntry);
    }
    
    this.writeToFile(logEntry);
  }

  warn(message, data = {}) {
    if (!this.shouldLog('warn')) return;
    
    const logEntry = this.formatLogEntry('WARN', message, data);
    
    if (this.options.logToConsole) {
      console.warn(logEntry);
    }
    
    this.writeToFile(logEntry);
  }

  error(message, data = {}) {
    if (!this.shouldLog('error')) return;
    
    const logEntry = this.formatLogEntry('ERROR', message, data);
    
    if (this.options.logToConsole) {
      console.error(logEntry);
    }
    
    this.writeToFile(logEntry);
  }

  // Specialized method for user activity logging
  userActivity(eventType, data = {}) {
    const logEntry = this.formatLogEntry('USER_ACTIVITY', eventType, data);
    
    // Don't log user activity to the console
    /*
    if (this.options.logToConsole) {
      console.log(`[USER_ACTIVITY] ${logEntry}`);
    }
    */
    
    this.writeToFile(logEntry, true);
  }
}

// Create and export default logger instance
const defaultLogger = new Logger({
  logToConsole: true,
  logToFile: true,
  minLevel: process.env.NODE_ENV === 'production' ? 'info' : 'debug'
});

module.exports = {
  Logger,
  logger: defaultLogger
};